module ActiveRedis
  module Save
    #TODO: add if valid?
    def save(options={})
      unless options[:validate] == false
        return false if invalid?(save_context)
      end

      if dirty?
        case save_context
        when :create
          perform_create
        when :update
          perform_update
        end
        true
      else
        false
      end
    end

    def new_record?
      !!@new_record
    end

    private

    def perform_save
      run_callbacks :save do
        self.class.redis.pipelined do
          self.changes.each do |key, value|
            if value.last.nil?
              send("#{key}_object").delete
            else
              send("set_#{key}", value)
            end
          end
        end
        @new_record = false
        @changed_attributes.clear
      end
    end

    def perform_update
      run_callbacks :update do
        perform_save
      end
    end

    def perform_create
      #TODO: check if no object exists under that id
      @id ||= SecureRandom.uuid().delete('-')
      run_callbacks :create do
        perform_save
      end
    end

    def save_context
      if new_record?
        :create
      else
        :update
      end
    end
  end
end